#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2006-12-18 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


def positionEncode(params, validate=False):
	'''Create a position binary message payload to pack into an AIS Msg position.

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 1 (field automatically set to "1")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - NavigationStatus(uint): What is the vessel doing
	  - ROT(int): RateOfTurn
	  - SOG(udecimal): Speed over ground
	  - PositionAccuracy(uint): Accuracy of positioning fixes
	  - Position_longitude(decimal): Location of the vessel  East West location
	  - Position_latitude(decimal): Location of the vessel  North South location
	  - COG(udecimal): Course over ground
	  - TrueHeading(uint): True heading (relative to true North)
	  - TimeStamp(uint): UTC second when the report was generated
	  - RegionalReserved(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - RAIM(bool): Receiver autonomous integrity monitoring flag
	  - syncstate(uint): Sycronization state
	  - slotoffset(uint): In what slot will the next transmission occur. BROKEN
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=1),6))
	if 'RepeatIndicator' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['RepeatIndicator']),2))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),2))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['UserID']),30))
	if 'NavigationStatus' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['NavigationStatus']),4))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=15),4))
	if 'ROT' in params:
		bvList.append(binary.bvFromSignedInt(params['ROT'],8))
	else:
		bvList.append(binary.bvFromSignedInt(-128,8))
	if 'SOG' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['SOG'])*Decimal('10')))),10))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(1023)),10))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['PositionAccuracy']),1))
	if 'Position_longitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['Position_longitude'])*Decimal('600000')),28))
	else:
		bvList.append(binary.bvFromSignedInt(108600000,28))
	if 'Position_latitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['Position_latitude'])*Decimal('600000')),27))
	else:
		bvList.append(binary.bvFromSignedInt(54600000,27))
	if 'COG' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['COG'])*Decimal('10')))),12))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(3600)),12))
	if 'TrueHeading' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['TrueHeading']),9))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=511),9))
	if 'TimeStamp' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['TimeStamp']),6))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=60),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),1))
	if params["RAIM"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['syncstate']),2))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['slotoffset']),14))

	return binary.joinBV(bvList)

def positionDecode(bv, validate=False):
	'''Unpack a position message 

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 1 (field automatically set to "1")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - NavigationStatus(uint): What is the vessel doing
	  - ROT(int): RateOfTurn
	  - SOG(udecimal): Speed over ground
	  - PositionAccuracy(uint): Accuracy of positioning fixes
	  - Position_longitude(decimal): Location of the vessel  East West location
	  - Position_latitude(decimal): Location of the vessel  North South location
	  - COG(udecimal): Course over ground
	  - TrueHeading(uint): True heading (relative to true North)
	  - TimeStamp(uint): UTC second when the report was generated
	  - RegionalReserved(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - RAIM(bool): Receiver autonomous integrity monitoring flag
	  - syncstate(uint): Sycronization state
	  - slotoffset(uint): In what slot will the next transmission occur. BROKEN
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['MessageID']=1
	r['RepeatIndicator']=int(bv[6:8])
	r['UserID']=int(bv[8:38])
	r['NavigationStatus']=int(bv[38:42])
	r['ROT']=binary.signedIntFromBV(bv[42:50])
	r['SOG']=Decimal(int(bv[50:60]))/Decimal('10')
	r['PositionAccuracy']=int(bv[60:61])
	r['Position_longitude']=Decimal(binary.signedIntFromBV(bv[61:89]))/Decimal('600000')
	r['Position_latitude']=Decimal(binary.signedIntFromBV(bv[89:116]))/Decimal('600000')
	r['COG']=Decimal(int(bv[116:128]))/Decimal('10')
	r['TrueHeading']=int(bv[128:137])
	r['TimeStamp']=int(bv[137:143])
	r['RegionalReserved']=0
	r['Spare']=0
	r['RAIM']=bool(int(bv[148:149]))
	r['syncstate']=int(bv[149:151])
	r['slotoffset']=int(bv[151:165])
	return r

def positionPrint(params, out=sys.stdout):
	'''Print a position message to stdout.

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 1 (field automatically set to "1")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - NavigationStatus(uint): What is the vessel doing
	  - ROT(int): RateOfTurn
	  - SOG(udecimal): Speed over ground
	  - PositionAccuracy(uint): Accuracy of positioning fixes
	  - Position_longitude(decimal): Location of the vessel  East West location
	  - Position_latitude(decimal): Location of the vessel  North South location
	  - COG(udecimal): Course over ground
	  - TrueHeading(uint): True heading (relative to true North)
	  - TimeStamp(uint): UTC second when the report was generated
	  - RegionalReserved(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - RAIM(bool): Receiver autonomous integrity monitoring flag
	  - syncstate(uint): Sycronization state
	  - slotoffset(uint): In what slot will the next transmission occur. BROKEN
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	out.write("position:\n")
	out.write("	MessageID:           "+str(params['MessageID'])+"\n")
	out.write("	RepeatIndicator:     "+str(params['RepeatIndicator'])+"\n")
	out.write("	UserID:              "+str(params['UserID'])+"\n")
	out.write("	NavigationStatus:    "+str(params['NavigationStatus'])+"\n")
	out.write("	ROT:                 "+str(params['ROT'])+"\n")
	out.write("	SOG:                 "+str(params['SOG'])+"\n")
	out.write("	PositionAccuracy:    "+str(params['PositionAccuracy'])+"\n")
	out.write("	Position_longitude:  "+str(params['Position_longitude'])+"\n")
	out.write("	Position_latitude:   "+str(params['Position_latitude'])+"\n")
	out.write("	COG:                 "+str(params['COG'])+"\n")
	out.write("	TrueHeading:         "+str(params['TrueHeading'])+"\n")
	out.write("	TimeStamp:           "+str(params['TimeStamp'])+"\n")
	out.write("	RegionalReserved:    "+str(params['RegionalReserved'])+"\n")
	out.write("	Spare:               "+str(params['Spare'])+"\n")
	out.write("	RAIM:                "+str(params['RAIM'])+"\n")
	out.write("	syncstate:           "+str(params['syncstate'])+"\n")
	out.write("	slotoffset:          "+str(params['slotoffset'])+"\n")

	return # Nothing to return



######################################################################
# UNIT TESTING
######################################################################
import unittest
def positionTestParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['MessageID'] = 1
	params['RepeatIndicator'] = 1
	params['UserID'] = 1193046
	params['NavigationStatus'] = 3
	params['ROT'] = -2
	params['SOG'] = Decimal('101.9')
	params['PositionAccuracy'] = 1
	params['Position_longitude'] = Decimal('-122.16328055555556')
	params['Position_latitude'] = Decimal('37.424458333333334')
	params['COG'] = Decimal('34.5')
	params['TrueHeading'] = 41
	params['TimeStamp'] = 35
	params['RegionalReserved'] = 0
	params['Spare'] = 0
	params['RAIM'] = False
	params['syncstate'] = 2
	params['slotoffset'] = 1221

	return params

class Testposition(unittest.TestCase):
	'''Uses the testvalue tag text from each type to build a test case for the position message'''
	def testEncodeDecode(self):
	
		params = positionTestParams()
		bits = positionEncode(params)
		r = positionDecode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['MessageID'],params['MessageID'])
		self.failUnlessEqual(r['RepeatIndicator'],params['RepeatIndicator'])
		self.failUnlessEqual(r['UserID'],params['UserID'])
		self.failUnlessEqual(r['NavigationStatus'],params['NavigationStatus'])
		self.failUnlessEqual(r['ROT'],params['ROT'])
		self.failUnlessAlmostEqual(r['SOG'],params['SOG'],1)
		self.failUnlessEqual(r['PositionAccuracy'],params['PositionAccuracy'])
		self.failUnlessAlmostEqual(r['Position_longitude'],params['Position_longitude'],5)
		self.failUnlessAlmostEqual(r['Position_latitude'],params['Position_latitude'],5)
		self.failUnlessAlmostEqual(r['COG'],params['COG'],7)
		self.failUnlessEqual(r['TrueHeading'],params['TrueHeading'])
		self.failUnlessEqual(r['TimeStamp'],params['TimeStamp'])
		self.failUnlessEqual(r['RegionalReserved'],params['RegionalReserved'])
		self.failUnlessEqual(r['Spare'],params['Spare'])
		self.failUnlessEqual(r['RAIM'],params['RAIM'])
		self.failUnlessEqual(r['syncstate'],params['syncstate'])
		self.failUnlessEqual(r['slotoffset'],params['slotoffset'])

############################################################
if __name__=='__main__':

    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options]",
			    version="%prog "+__version__)

    #sys.exit('EARLY EXIT - FIX: remove')
    myparser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
                        help='run the documentation tests')
    myparser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
                        help='run the unit tests')
    myparser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
                        help='Make the test output verbose')

    (options,args) = myparser.parse_args()
    success=True

    if options.doctest:
	import os; print os.path.basename(sys.argv[0]), 'doctests ...',
	sys.argv= [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')
	import doctest
	numfail,numtests=doctest.testmod()
	if numfail==0: print 'ok'
	else: 
	    print 'FAILED'
	    success=False

    if not success:
	sys.exit('Something Failed')

    del success # Hide success from epydoc

    if options.unittest:
	sys.argv = [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')
	unittest.main()
