#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2006-12-18 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


def sls_waterlevelEncode(params, validate=False):
	'''Create a sls_waterlevel binary message payload to pack into an AIS Msg sls_waterlevel.

	Fields in params:
	  - time_month(uint): Time tag of measurement  month 1..12
	  - time_day(uint): Time tag of measurement  day of the month 1..31
	  - time_hour(uint): Time tag of measurement  UTC hours 0..23
	  - time_min(uint): Time tag of measurement  minutes
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - pos_longitude(decimal): Location of measurement  East West location
	  - pos_latitude(decimal): Location of measurement  North South location
	  - type(uint): How to interpret the water level
	  - waterlevel(int): Water level in centimeters
	  - datum(uint): What reference datum applies to the value
	  - reserved(int): Reserved bits for future use (field automatically set to "0")
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['time_month']),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['time_day']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['time_hour']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['time_min']),6))
	if 'stationid' in params:
		bvList.append(aisstring.encode(params['stationid'],42))
	else:
		bvList.append(aisstring.encode('@@@@@@@',42))
	if 'pos_longitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['pos_longitude'])*Decimal('60000')),25))
	else:
		bvList.append(binary.bvFromSignedInt(10860000,25))
	if 'pos_latitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['pos_latitude'])*Decimal('60000')),24))
	else:
		bvList.append(binary.bvFromSignedInt(5460000,24))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['type']),1))
	if 'waterlevel' in params:
		bvList.append(binary.bvFromSignedInt(params['waterlevel'],16))
	else:
		bvList.append(binary.bvFromSignedInt(-32768,16))
	if 'datum' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['datum']),5))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=31),5))
	bvList.append(binary.bvFromSignedInt(0,14))

	return binary.joinBV(bvList)

def sls_waterlevelDecode(bv, validate=False):
	'''Unpack a sls_waterlevel message 

	Fields in params:
	  - time_month(uint): Time tag of measurement  month 1..12
	  - time_day(uint): Time tag of measurement  day of the month 1..31
	  - time_hour(uint): Time tag of measurement  UTC hours 0..23
	  - time_min(uint): Time tag of measurement  minutes
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - pos_longitude(decimal): Location of measurement  East West location
	  - pos_latitude(decimal): Location of measurement  North South location
	  - type(uint): How to interpret the water level
	  - waterlevel(int): Water level in centimeters
	  - datum(uint): What reference datum applies to the value
	  - reserved(int): Reserved bits for future use (field automatically set to "0")
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['time_month']=int(bv[0:4])
	r['time_day']=int(bv[4:9])
	r['time_hour']=int(bv[9:14])
	r['time_min']=int(bv[14:20])
	r['stationid']=aisstring.decode(bv[20:62])
	r['pos_longitude']=Decimal(binary.signedIntFromBV(bv[62:87]))/Decimal('60000')
	r['pos_latitude']=Decimal(binary.signedIntFromBV(bv[87:111]))/Decimal('60000')
	r['type']=int(bv[111:112])
	r['waterlevel']=binary.signedIntFromBV(bv[112:128])
	r['datum']=int(bv[128:133])
	r['reserved']=0
	return r

def decodetime_month(bv, validate=False):
	r={};	r['time_month']=int(bv[0:4])
	return(r['time_month'])

def decodetime_day(bv, validate=False):
	r={};	r['time_day']=int(bv[4:9])
	return(r['time_day'])

def decodetime_hour(bv, validate=False):
	r={};	r['time_hour']=int(bv[9:14])
	return(r['time_hour'])

def decodetime_min(bv, validate=False):
	r={};	r['time_min']=int(bv[14:20])
	return(r['time_min'])

def decodestationid(bv, validate=False):
	r={};	r['stationid']=aisstring.decode(bv[20:62])
	return(r['stationid'])

def decodepos_longitude(bv, validate=False):
	r={};	r['pos_longitude']=Decimal(binary.signedIntFromBV(bv[62:87]))/Decimal('60000')
	return(r['pos_longitude'])

def decodepos_latitude(bv, validate=False):
	r={};	r['pos_latitude']=Decimal(binary.signedIntFromBV(bv[87:111]))/Decimal('60000')
	return(r['pos_latitude'])

def decodetype(bv, validate=False):
	r={};	r['type']=int(bv[111:112])
	return(r['type'])

def decodewaterlevel(bv, validate=False):
	r={};	r['waterlevel']=binary.signedIntFromBV(bv[112:128])
	return(r['waterlevel'])

def decodedatum(bv, validate=False):
	r={};	r['datum']=int(bv[128:133])
	return(r['datum'])

def decodereserved(bv, validate=False):
	r={};	r['reserved']=0
	return(r['reserved'])

def sls_waterlevelPrint(params, out=sys.stdout):
	'''Print a sls_waterlevel message to stdout.

	Fields in params:
	  - time_month(uint): Time tag of measurement  month 1..12
	  - time_day(uint): Time tag of measurement  day of the month 1..31
	  - time_hour(uint): Time tag of measurement  UTC hours 0..23
	  - time_min(uint): Time tag of measurement  minutes
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - pos_longitude(decimal): Location of measurement  East West location
	  - pos_latitude(decimal): Location of measurement  North South location
	  - type(uint): How to interpret the water level
	  - waterlevel(int): Water level in centimeters
	  - datum(uint): What reference datum applies to the value
	  - reserved(int): Reserved bits for future use (field automatically set to "0")
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	out.write("sls_waterlevel:\n")
	out.write("	time_month:     "+str(params['time_month'])+"\n")
	out.write("	time_day:       "+str(params['time_day'])+"\n")
	out.write("	time_hour:      "+str(params['time_hour'])+"\n")
	out.write("	time_min:       "+str(params['time_min'])+"\n")
	out.write("	stationid:      "+str(params['stationid'])+"\n")
	out.write("	pos_longitude:  "+str(params['pos_longitude'])+"\n")
	out.write("	pos_latitude:   "+str(params['pos_latitude'])+"\n")
	out.write("	type:           "+str(params['type'])+"\n")
	out.write("	waterlevel:     "+str(params['waterlevel'])+"\n")
	out.write("	datum:          "+str(params['datum'])+"\n")
	out.write("	reserved:       "+str(params['reserved'])+"\n")

	return # Nothing to return



######################################################################
# UNIT TESTING
######################################################################
import unittest
def sls_waterlevelTestParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['time_month'] = 2
	params['time_day'] = 28
	params['time_hour'] = 23
	params['time_min'] = 45
	params['stationid'] = 'A234567'
	params['pos_longitude'] = Decimal('-122.16328')
	params['pos_latitude'] = Decimal('37.42446')
	params['type'] = 0
	params['waterlevel'] = -97
	params['datum'] = 0
	params['reserved'] = 0

	return params

class Testsls_waterlevel(unittest.TestCase):
	'''Uses the testvalue tag text from each type to build a test case for the sls_waterlevel message'''
	def testEncodeDecode(self):
	
		params = sls_waterlevelTestParams()
		bits = sls_waterlevelEncode(params)
		r = sls_waterlevelDecode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['time_month'],params['time_month'])
		self.failUnlessEqual(r['time_day'],params['time_day'])
		self.failUnlessEqual(r['time_hour'],params['time_hour'])
		self.failUnlessEqual(r['time_min'],params['time_min'])
		self.failUnlessEqual(r['stationid'],params['stationid'])
		self.failUnlessAlmostEqual(r['pos_longitude'],params['pos_longitude'],4)
		self.failUnlessAlmostEqual(r['pos_latitude'],params['pos_latitude'],4)
		self.failUnlessEqual(r['type'],params['type'])
		self.failUnlessEqual(r['waterlevel'],params['waterlevel'])
		self.failUnlessEqual(r['datum'],params['datum'])
		self.failUnlessEqual(r['reserved'],params['reserved'])

############################################################
if __name__=='__main__':

    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options]",
			    version="%prog "+__version__)

    #sys.exit('EARLY EXIT - FIX: remove')
    myparser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
                        help='run the documentation tests')
    myparser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
                        help='run the unit tests')
    myparser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
                        help='Make the test output verbose')

    (options,args) = myparser.parse_args()
    success=True

    if options.doctest:
	import os; print os.path.basename(sys.argv[0]), 'doctests ...',
	sys.argv= [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')
	import doctest
	numfail,numtests=doctest.testmod()
	if numfail==0: print 'ok'
	else: 
	    print 'FAILED'
	    success=False

    if not success:
	sys.exit('Something Failed')

    del success # Hide success from epydoc

    if options.unittest:
	sys.argv = [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')
	unittest.main()
